package serverClient;
import java.io.*;
import java.net.*;
import java.util.*;
import stones.*;
import commands.*;
import GUIPack.*;
/**
 *  This class handles the communication with the server and is watching over several conditions.
 * @author Winkler
 */
public class Client_fromServer extends Thread 
{
	//public C_common knowsAll;

	//gui stuff
	//private String serverHost;
	private String playerName;
	public TetPanel tetrisPanel;
	private NextStoneLabel nextStoneShowLabel;
	public C_GUI gui;
	//public HighScore hs;

	//client-server stuff
	private Socket so;
	public ObjectOutputStream objectToServer;
	private ObjectInputStream objectFromServer;


	private int numOfLines, lineTest;
	public static boolean kill = false;
	private int score = 0;
	public boolean paintStone = false;

	//Stone list
	private ArrayList<S_Stone> stoneList = new ArrayList<S_Stone>();

	public S_Stone nextStone, actualStone;

	public Client_fromServer(String serverHost, String playerName, TetPanel tetrisPanel, NextStoneLabel nextStoneShowLabel, C_GUI gui)
	{

		this.playerName = playerName;
		this.tetrisPanel = tetrisPanel;
		this.nextStoneShowLabel = nextStoneShowLabel;
		this.gui = gui;


		try
		{

			so = new Socket(serverHost, 16322);
			objectToServer = new ObjectOutputStream(so.getOutputStream());
			objectFromServer = new ObjectInputStream(so.getInputStream());

		}
		catch (Exception e)
		{
			System.out.println("Client_fromServer connection exception");
		}
	}

	/**
	 * The run()-method is started when this thread is called on, it has the sole purpose to start the go()-method.
	 */
	public void run()
	{
		try
		{
			go();
		}
		catch(Exception e){};
	}

	/**
	 *  This method contains the most important tasks of the class.
	 *  Such as sending the player-name to the server,
	 *  managing the correct handling of the incoming stone-objects,
	 *  removing full lines and requesting the server to set new scores,
	 *  checking if this player is game over,
	 *  @exception IOException is thrown on errors in network communication
	 */

	public void go() throws IOException
	{
		//String serverOutput;
		Object serverObjectOutput;



		try
		{
			//Das erste was an den Server geschickt wird ist der Spielername (es ist das erste was er erwartet).
			objectToServer.writeObject(playerName);
			objectToServer.flush();

			while(!kill)
			{
				//es wird darauf gewartet, dass der Server ein Objekt schickt. Ist eines da geht es weiter...
				serverObjectOutput = objectFromServer.readObject();

				//string printout
				//System.out.println("C_fS - stoneList.size(): "+stoneList.size());

				//Es wird zur Vaterklasse aller Commandos gecastet.
				S_Command commandObject = (S_Command)serverObjectOutput;
				//System.out.println("Command in C_fS: "+commandObject.toString()+" stoneList.size(): "+stoneList.size());
				//Die execute-Methode ist im Vater nicht ausimplementiert und wird daher implements spezifischen Kind ausgefuehrt.
				commandObject.execute(this);


				//=================================================
				//Hier wird das eingegangene Steinobjekt verarbeitet
				if(stoneList.size() > 2 && paintStone)
				{	
					//sollten beide verwalteten Steine null sein, dann f�lle erstmal den der auf's Spielfeld geschreiben wird
					if (nextStone==null && actualStone==null)
					{
						actualStone = fillStone();
						actualStone.tetrisPanel = tetrisPanel;
						tetrisPanel.stone = actualStone;

						nextStone = fillStone();
						nextStoneShowLabel.stone = nextStone;
					}
					//ist nur der Spielfeldstein null, dann mache den Vorschaustein zum Spielfeldstein und den eingehenden zum Vorschaustein
					else if(actualStone==null)
					{
						actualStone = nextStone;
						actualStone.tetrisPanel = tetrisPanel;
						tetrisPanel.stone = actualStone;

						nextStone = fillStone();
						nextStoneShowLabel.stone = nextStone;

					}
					//ist nur der Vorschaustein null, dann f�lle ihn mit dem eingehenden
					else if(nextStone==null)
					{
						nextStone = fillStone();
						nextStoneShowLabel.stone = nextStone;
					}

					//nachdem der/die jeweiligen Stein(e) ersetzt wurden, muessen Vorschau und Spielfeld aktualisiert werden
					nextStoneShowLabel.repaint();
					tetrisPanel.repaint();	
				}

				if(stoneList.size() < 3)
				{
					sendGetStoneCommand();
				}

				lineChecker();

				gameOverCheck();
			}
		}
		catch(Exception e)
		{
			//falls eine Exception auftritt
			System.out.print("Client_fromServer Exception >>> Client disconnected!! <<<\n");
		}

		try
		{
			//dem Server wird mitgeteilt, dass der Client Game Over ist.
			objectToServer.writeObject(new C_gameOverCommand());
			//der Client sendet (und fordert damit) nochmal einen aktuellen Punktestand an.
			objectToServer.writeObject(new C_newScoreCommand(score));

			//...darf dem Spiel weiter beiwohnen aber bekommt keine Steine mehr, sondern wird nur noch ueber Aenderungen bei den anderen Spielern informiert
			while(!false)
			{
				serverObjectOutput = objectFromServer.readObject();
				if (serverObjectOutput instanceof S_Command)
				{
					S_Command commandObject = (S_Command)serverObjectOutput;
					commandObject.execute(this);
				}
			}
		}
		catch(Exception e)
		{
			System.out.print(">>> Client disconnected!! <<<\n");
		}
	}

	public void lineChecker()
	{
		numOfLines = 0;
		//gibt an ob Linien zu entfernen sind
		lineTest = 1;

		//geht alle Zeilen des Spielfeldes durch...
		for(int z = 3; z < 23; z++)
		{
			//...ueberprueft die einzelnen Eintraege auf gefuellte Felder...
			for(int s = 2; s < 12; s++)
			{

				lineTest = lineTest * tetrisPanel.getTetMatAt(s,z).col;
			}
			//...und ist eine Linie voll...
			if(lineTest != 0)
			{
				//...zaehle die Linienanzahl hoch...
				numOfLines++;

				//...und r�cke alle Linien eins herunter...
				//...indem du von der vollen Linie nach oben hin...
				for(int zi = z; zi > 2; zi--)
				{
					//...alle Bloecke um 1 nach unten setzt
					for(int si = 2; si < 12; si++)
					{
						tetrisPanel.setTetMatAt(si, zi, tetrisPanel.getTetMatAt(si,zi-1));
						//tetrisPanel.tetMat[si][zi] = tetrisPanel.tetMat[si][zi-1];
					}
				}
				// wird wieder auf den Standard 1 gesetzt
				lineTest = 1;
				// Die abgeraeumten Linien addieren sich quadriert zum alten Punktestand hinzu
				score = score + (numOfLines * numOfLines * 10);
				// Der neue Punktestand wird an den Server gesandt

				try {
					objectToServer.writeObject(new C_newScoreCommand(score));
					objectToServer.flush();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
			else
			{
				// wird wieder auf den Standard 1 gesetzt
				lineTest = 1;
			}
		}
	}

	public void gameOverCheck()
	{
		for(int i = 2; i <  12; i++)
		{
			if(tetrisPanel.getTetMatAt(i,2).col!=0)
			{
				kill=true;
				tetrisPanel.gameOver=true;
			}
		}
	}

	public void copyStones(ArrayList<S_Stone> stones)
	{
		stoneList.addAll(stones);

		//System.out.println("copied Stones => stoneList.size() = "+stoneList.size());
	}

	public S_Stone fillStone()
	{
		S_Stone stone = null;

		if(stoneList.size() > 0){
			stone = stoneList.get(0);
			stoneList.remove(0);
			//System.out.println("fillStone()");
			return stone;
		}
		else
		{
			//System.out.println("fillStone() - stoneList has no Stone");
			return null;
		}
	}

	public void sendGetStoneCommand()
	{
		try 
		{
			objectToServer.writeObject(new C_getStoneCommand());
			objectToServer.flush();
		} catch (IOException e) 
		{
		}
	}
	
	public void setTimeLeft(String time)
	{
		gui.setTimeL(time);
	}
	
	public void setTimeLabel(String txt)
	{
		gui.setTimeLabel(txt);
	}
	
	public void setPartPlayer(String player)
	{
		gui.setPlayerL(player);
	}
}
